@@ -3,20 +3,26 @@ package Eutf2;
 #
 # Eutf2 - Run-time routines for UTF2.pm
 #
-# Copyright (c) 2008, 2009, 2010, 2011 INABA Hitoshi <ina@cpan.org>
+# Copyright (c) 2008, 2009, 2010, 2011, 2012 INABA Hitoshi <ina@cpan.org>
 #
 ######################################################################
 
 use 5.00503;
 use strict qw(subs vars);
 
+BEGIN {
+    if ($^X =~ m/ jperl /oxmsi) {
+        die "$0 need perl(not jperl) 5.00503 or later. (\$^X==$^X)";
+    }
+}
+
 # 12.3. Delaying use Until Runtime
 # in Chapter 12. Packages, Libraries, and Modules
 # of ISBN 0-596-00313-7 Perl Cookbook, 2nd Edition.
 # (and so on)
 
 BEGIN { eval q{ use vars qw($VERSION) } }
-$VERSION = sprintf '%d.%02d', q$Revision: 0.79 $ =~ m/(\d+)/xmsg;
+$VERSION = sprintf '%d.%02d', q$Revision: 0.80 $ =~ m/(\d+)/xmsg;
 
 BEGIN {
     my $PERL5LIB = __FILE__;
@@ -106,6 +112,10 @@ BEGIN {
 # in Chapter 29: Functions
 # of ISBN 0-596-00027-8 Programming Perl Third Edition.
 
+# P.863 flock
+# in Chapter 27: Functions
+# of ISBN 978-0-596-00492-7 Programming Perl 4th Edition.
+
 sub LOCK_SH() {1}
 sub LOCK_EX() {2}
 sub LOCK_UN() {8}
@@ -119,12 +129,6 @@ sub confess(@);
 
 my $__FILE__ = __FILE__;
 
-BEGIN {
-    if ($^X =~ m/ jperl /oxmsi) {
-        die "$0 need perl(not jperl) 5.00503 or later. (\$^X==$^X)";
-    }
-}
-
 my $your_char = q{(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[\x00-\xFF]};
 
 # regexp of character
@@ -378,6 +382,68 @@ sub UTF2::substr($$;$$);
 sub UTF2::index($$;$);
 sub UTF2::rindex($$;$);
 
+#
+# Character class
+#
+use vars qw(
+    @anchor
+    @dot
+    @dot_s
+    @eD
+    @eS
+    @eW
+    @eH
+    @eV
+    @eR
+    @eN
+    @not_alnum
+    @not_alpha
+    @not_ascii
+    @not_blank
+    @not_cntrl
+    @not_digit
+    @not_graph
+    @not_lower
+    @not_lower_i
+    @not_print
+    @not_punct
+    @not_space
+    @not_upper
+    @not_upper_i
+    @not_word
+    @not_xdigit
+    @eb
+    @eB
+);
+@{Eutf2::anchor}      = qr{\G(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF])*?};
+@{Eutf2::dot}         = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x0A])};
+@{Eutf2::dot_s}       = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF])};
+@{Eutf2::eD}          = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF0-9])};
+@{Eutf2::eS}          = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x09\x0A\x0C\x0D\x20])};
+@{Eutf2::eW}          = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF0-9A-Z_a-z])};
+@{Eutf2::eH}          = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x09\x20])};
+@{Eutf2::eV}          = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x0C\x0A\x0D])};
+@{Eutf2::eR}          = qr{(?:\x0D\x0A|[\x0A\x0D])};
+@{Eutf2::eN}          = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x0A])};
+@{Eutf2::not_alnum}   = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x30-\x39\x41-\x5A\x61-\x7A])};
+@{Eutf2::not_alpha}   = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x41-\x5A\x61-\x7A])};
+@{Eutf2::not_ascii}   = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x00-\x7F])};
+@{Eutf2::not_blank}   = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x09\x20])};
+@{Eutf2::not_cntrl}   = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x00-\x1F\x7F])};
+@{Eutf2::not_digit}   = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x30-\x39])};
+@{Eutf2::not_graph}   = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x21-\x7F])};
+@{Eutf2::not_lower}   = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x61-\x7A])};
+@{Eutf2::not_lower_i} = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF])};
+@{Eutf2::not_print}   = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x20-\x7F])};
+@{Eutf2::not_punct}   = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x21-\x2F\x3A-\x3F\x40\x5B-\x5F\x60\x7B-\x7E])};
+@{Eutf2::not_space}   = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x09\x0A\x0B\x0C\x0D\x20])};
+@{Eutf2::not_upper}   = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x41-\x5A])};
+@{Eutf2::not_upper_i} = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF])};
+@{Eutf2::not_word}    = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x30-\x39\x41-\x5A\x5F\x61-\x7A])};
+@{Eutf2::not_xdigit}  = qr{(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x30-\x39\x41-\x46\x61-\x66])};
+@{Eutf2::eb}          = qr{(?:\A(?=[0-9A-Z_a-z])|(?<=[\x00-\x2F\x40\x5B-\x5E\x60\x7B-\xFF])(?=[0-9A-Z_a-z])|(?<=[0-9A-Z_a-z])(?=[\x00-\x2F\x40\x5B-\x5E\x60\x7B-\xFF]|\z))};
+@{Eutf2::eB}          = qr{(?:(?<=[0-9A-Z_a-z])(?=[0-9A-Z_a-z])|(?<=[\x00-\x2F\x40\x5B-\x5E\x60\x7B-\xFF])(?=[\x00-\x2F\x40\x5B-\x5E\x60\x7B-\xFF]))};
+
 #
 # @ARGV wildcard globbing
 #
@@ -408,6 +474,10 @@ sub Eutf2::split(;$$$) {
     # in Chapter 29: Functions
     # of ISBN 0-596-00027-8 Programming Perl Third Edition.
 
+    # P.951 split
+    # in Chapter 27: Functions
+    # of ISBN 978-0-596-00492-7 Programming Perl 4th Edition.
+
     my $pattern = $_[0];
     my $string  = $_[1];
     my $limit   = $_[2];
@@ -727,9 +797,9 @@ sub classic_character_class($) {
     my($char) = @_;
 
     return {
-        '\D' => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF0-9])',
-        '\S' => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x09\x0A\x0C\x0D\x20])',
-        '\W' => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF0-9A-Z_a-z])',
+        '\D' => '@{Eutf2::eD}',
+        '\S' => '@{Eutf2::eS}',
+        '\W' => '@{Eutf2::eW}',
         '\d' => '[0-9]',
                  # \t  \n  \f  \r space
         '\s' => '[\x09\x0A\x0C\x0D\x20]',
@@ -738,16 +808,22 @@ sub classic_character_class($) {
         '\X' => 'X',
 
         # \h \v \H \V
-        #
+
         # P.114 Character Class Shortcuts
         # in Chapter 7: In the World of Regular Expressions
         # of ISBN 978-0-596-52010-6 Learning Perl, Fifth Edition
 
-        '\H' => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x09\x20])',
-        '\V' => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x0C\x0A\x0D])',
+        # P.196 Table 5-9. Alphanumeric regex metasymbols
+        # in Chapter 5. Pattern Matching
+        # of ISBN 978-0-596-00492-7 Programming Perl 4th Edition.
+
+        # (and so on)
+
+        '\H' => '@{Eutf2::eH}',
+        '\V' => '@{Eutf2::eV}',
         '\h' => '[\x09\x20]',
         '\v' => '[\x0C\x0A\x0D]',
-        '\R' => '(?:\x0D\x0A|[\x0A\x0D])',
+        '\R' => '@{Eutf2::eR}',
 
         # \N
         #
@@ -755,19 +831,23 @@ sub classic_character_class($) {
         # Character Classes and other Special Escapes
         # Any character but \n (experimental). Not affected by /s modifier
 
-        '\N' => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x0A])',
+        '\N' => '@{Eutf2::eN}',
 
         # \b \B
-        #
+
         # P.131 Word boundaries: \b, \B, \<, \>, ...
         # in Chapter 3: Overview of Regular Expression Features and Flavors
         # of ISBN 0-596-00289-0 Mastering Regular Expressions, Second edition
 
+        # P.219 Boundaries: The \b and \B Assertions
+        # in Chapter 5: Pattern Matching
+        # of ISBN 978-0-596-00492-7 Programming Perl 4th Edition.
+
         # '\b' => '(?:(?<=\A|\W)(?=\w)|(?<=\w)(?=\W|\z))',
-        '\b' => '(?:\A(?=[0-9A-Z_a-z])|(?<=[\x00-\x2F\x40\x5B-\x5E\x60\x7B-\xFF])(?=[0-9A-Z_a-z])|(?<=[0-9A-Z_a-z])(?=[\x00-\x2F\x40\x5B-\x5E\x60\x7B-\xFF]|\z))',
+        '\b' => '@{Eutf2::eb}',
 
         # '\B' => '(?:(?<=\w)(?=\w)|(?<=\W)(?=\W))',
-        '\B' => '(?:(?<=[0-9A-Z_a-z])(?=[0-9A-Z_a-z])|(?<=[\x00-\x2F\x40\x5B-\x5E\x60\x7B-\xFF])(?=[\x00-\x2F\x40\x5B-\x5E\x60\x7B-\xFF]))',
+        '\B' => '@{Eutf2::eB}',
 
     }->{$char} || '';
 }
@@ -916,7 +996,7 @@ sub _charlist_tr {
         }
     }
 
-    # join separated multiple octet
+    # join separated multiple-octet
     @char = join('',@char) =~ m/\G (\\-|$q_char) /oxmsg;
 
     # unescape '-'
@@ -941,53 +1021,53 @@ sub _charlist_tr {
             croak "$0: invalid [] range \"\\x" . unpack('H*',$char[$i-1]) . '-\\x' . unpack('H*',$char[$i+1]) . '" in regexp';
         }
 
-        # range of multiple octet code
+        # range of multiple-octet code
         if (length($char[$i-1]) == 1) {
             if (length($char[$i+1]) == 1) {
-                push @range, grep {($char[$i-1] le $_) and ($_ le $char[$i+1])} &chars1();
+                push @range, grep {($char[$i-1] le $_) and ($_ le $char[$i+1])} chars1();
             }
             elsif (length($char[$i+1]) == 2) {
-                push @range, grep {$char[$i-1] le $_}                           &chars1();
-                push @range, grep {$_ le $char[$i+1]}                           &chars2();
+                push @range, grep {$char[$i-1] le $_}                           chars1();
+                push @range, grep {$_ le $char[$i+1]}                           chars2();
             }
             elsif (length($char[$i+1]) == 3) {
-                push @range, grep {$char[$i-1] le $_}                           &chars1();
-                push @range,                                                    &chars2();
-                push @range, grep {$_ le $char[$i+1]}                           &chars3();
+                push @range, grep {$char[$i-1] le $_}                           chars1();
+                push @range,                                                    chars2();
+                push @range, grep {$_ le $char[$i+1]}                           chars3();
             }
             elsif (length($char[$i+1]) == 4) {
-                push @range, grep {$char[$i-1] le $_}                           &chars1();
-                push @range,                                                    &chars2();
-                push @range,                                                    &chars3();
-                push @range, grep {$_ le $char[$i+1]}                           &chars4();
+                push @range, grep {$char[$i-1] le $_}                           chars1();
+                push @range,                                                    chars2();
+                push @range,                                                    chars3();
+                push @range, grep {$_ le $char[$i+1]}                           chars4();
             }
         }
         elsif (length($char[$i-1]) == 2) {
             if (length($char[$i+1]) == 2) {
-                push @range, grep {($char[$i-1] le $_) and ($_ le $char[$i+1])} &chars2();
+                push @range, grep {($char[$i-1] le $_) and ($_ le $char[$i+1])} chars2();
             }
             elsif (length($char[$i+1]) == 3) {
-                push @range, grep {$char[$i-1] le $_}                           &chars2();
-                push @range, grep {$_ le $char[$i+1]}                           &chars3();
+                push @range, grep {$char[$i-1] le $_}                           chars2();
+                push @range, grep {$_ le $char[$i+1]}                           chars3();
             }
             elsif (length($char[$i+1]) == 4) {
-                push @range, grep {$char[$i-1] le $_}                           &chars2();
-                push @range,                                                    &chars3();
-                push @range, grep {$_ le $char[$i+1]}                           &chars4();
+                push @range, grep {$char[$i-1] le $_}                           chars2();
+                push @range,                                                    chars3();
+                push @range, grep {$_ le $char[$i+1]}                           chars4();
             }
         }
         elsif (length($char[$i-1]) == 3) {
             if (length($char[$i+1]) == 3) {
-                push @range, grep {($char[$i-1] le $_) and ($_ le $char[$i+1])} &chars3();
+                push @range, grep {($char[$i-1] le $_) and ($_ le $char[$i+1])} chars3();
             }
             elsif (length($char[$i+1]) == 4) {
-                push @range, grep {$char[$i-1] le $_}                           &chars3();
-                push @range, grep {$_ le $char[$i+1]}                           &chars4();
+                push @range, grep {$char[$i-1] le $_}                           chars3();
+                push @range, grep {$_ le $char[$i+1]}                           chars4();
             }
         }
         elsif (length($char[$i-1]) == 4) {
             if (length($char[$i+1]) == 4) {
-                push @range, grep {($char[$i-1] le $_) and ($_ le $char[$i+1])} &chars4();
+                push @range, grep {($char[$i-1] le $_) and ($_ le $char[$i+1])} chars4();
             }
         }
 
@@ -1024,7 +1104,7 @@ sub _octets {
             }
         }
 
-        # not ignore case or one of multiple octet
+        # not ignore case or one of multiple-octet
         else {
             if ($a == $z) {
                 return sprintf('\x%02X',          $a);
@@ -1099,7 +1179,7 @@ sub _octets {
         return "(?:$octets1|$octets2)";
     }
 
-    # multiple octet code
+    # multiple-octet code
     else {
         my(undef,$aa) = unpack 'Ca*', $_[0];
         my(undef,$zz) = unpack 'Ca*', $_[1];
@@ -1111,29 +1191,29 @@ sub _octets {
         }
         elsif (($a+1) == $z) {
             return '(?:' . join('|',
-                sprintf('\x%02X%s',         $a,         _octets($length-1,$aa,                &maxchar($length-1),$modifier)),
-                sprintf('\x%02X%s',              $z,    _octets($length-1,&minchar($length-1),$zz,                $modifier)),
+                sprintf('\x%02X%s',         $a,         _octets($length-1,$aa,                maxchar($length-1),$modifier)),
+                sprintf('\x%02X%s',              $z,    _octets($length-1,minchar($length-1),$zz,                $modifier)),
             ) . ')';
         }
         elsif (($a+2) == $z) {
             return '(?:' . join('|',
-                sprintf('\x%02X%s',         $a,         _octets($length-1,$aa,                &maxchar($length-1),$modifier)),
-                sprintf('\x%02X%s',         $a+1,       _octets($length-1,&minchar($length-1),&maxchar($length-1),$modifier)),
-                sprintf('\x%02X%s',              $z,    _octets($length-1,&minchar($length-1),$zz,                $modifier)),
+                sprintf('\x%02X%s',         $a,         _octets($length-1,$aa,               maxchar($length-1),$modifier)),
+                sprintf('\x%02X%s',         $a+1,       _octets($length-1,minchar($length-1),maxchar($length-1),$modifier)),
+                sprintf('\x%02X%s',              $z,    _octets($length-1,minchar($length-1),$zz,               $modifier)),
             ) . ')';
         }
         elsif (($a+3) == $z) {
             return '(?:' . join('|',
-                sprintf('\x%02X%s',         $a,         _octets($length-1,$aa,                &maxchar($length-1),$modifier)),
-                sprintf('[\x%02X\x%02X]%s', $a+1,$z-1,  _octets($length-1,&minchar($length-1),&maxchar($length-1),$modifier)),
-                sprintf('\x%02X%s',              $z,    _octets($length-1,&minchar($length-1),$zz,                $modifier)),
+                sprintf('\x%02X%s',         $a,         _octets($length-1,$aa,               maxchar($length-1),$modifier)),
+                sprintf('[\x%02X\x%02X]%s', $a+1,$z-1,  _octets($length-1,minchar($length-1),maxchar($length-1),$modifier)),
+                sprintf('\x%02X%s',              $z,    _octets($length-1,minchar($length-1),$zz,               $modifier)),
             ) . ')';
         }
         else {
             return '(?:' . join('|',
-                sprintf('\x%02X%s',          $a,        _octets($length-1,$aa,                &maxchar($length-1),$modifier)),
-                sprintf('[\x%02X-\x%02X]%s', $a+1,$z-1, _octets($length-1,&minchar($length-1),&maxchar($length-1),$modifier)),
-                sprintf('\x%02X%s',               $z,   _octets($length-1,&minchar($length-1),$zz,                $modifier)),
+                sprintf('\x%02X%s',          $a,        _octets($length-1,$aa,               maxchar($length-1),$modifier)),
+                sprintf('[\x%02X-\x%02X]%s', $a+1,$z-1, _octets($length-1,minchar($length-1),maxchar($length-1),$modifier)),
+                sprintf('\x%02X%s',               $z,   _octets($length-1,minchar($length-1),$zz,               $modifier)),
             ) . ')';
         }
     }
@@ -1211,15 +1291,15 @@ sub _charlist {
                 '\d' => '[0-9]',
                 '\s' => '[\x09\x0A\x0C\x0D\x20]',
                 '\w' => '[0-9A-Z_a-z]',
-                '\D' => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF0-9])',
-                '\S' => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x09\x0A\x0C\x0D\x20])',
-                '\W' => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF0-9A-Z_a-z])',
+                '\D' => '@{Eutf2::eD}',
+                '\S' => '@{Eutf2::eS}',
+                '\W' => '@{Eutf2::eW}',
 
-                '\H' => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x09\x20])',
-                '\V' => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x0C\x0A\x0D])',
+                '\H' => '@{Eutf2::eH}',
+                '\V' => '@{Eutf2::eV}',
                 '\h' => '[\x09\x20]',
                 '\v' => '[\x0C\x0A\x0D]',
-                '\R' => '(?:\x0D\x0A|[\x0A\x0D])',
+                '\R' => '@{Eutf2::eR}',
 
             }->{$1};
         }
@@ -1230,8 +1310,8 @@ sub _charlist {
 
                 '[:lower:]'   => '[\x41-\x5A\x61-\x7A]',
                 '[:upper:]'   => '[\x41-\x5A\x61-\x7A]',
-                '[:^lower:]'  => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF])',
-                '[:^upper:]'  => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF])',
+                '[:^lower:]'  => '@{Eutf2::not_lower_i}',
+                '[:^upper:]'  => '@{Eutf2::not_upper_i}',
 
             }->{$1};
         }
@@ -1252,20 +1332,20 @@ sub _charlist {
                 '[:upper:]'   => '[\x41-\x5A]',
                 '[:word:]'    => '[\x30-\x39\x41-\x5A\x5F\x61-\x7A]',
                 '[:xdigit:]'  => '[\x30-\x39\x41-\x46\x61-\x66]',
-                '[:^alnum:]'  => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x30-\x39\x41-\x5A\x61-\x7A])',
-                '[:^alpha:]'  => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x41-\x5A\x61-\x7A])',
-                '[:^ascii:]'  => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x00-\x7F])',
-                '[:^blank:]'  => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x09\x20])',
-                '[:^cntrl:]'  => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x00-\x1F\x7F])',
-                '[:^digit:]'  => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x30-\x39])',
-                '[:^graph:]'  => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x21-\x7F])',
-                '[:^lower:]'  => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x61-\x7A])',
-                '[:^print:]'  => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x20-\x7F])',
-                '[:^punct:]'  => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x21-\x2F\x3A-\x3F\x40\x5B-\x5F\x60\x7B-\x7E])',
-                '[:^space:]'  => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x09\x0A\x0B\x0C\x0D\x20])',
-                '[:^upper:]'  => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x41-\x5A])',
-                '[:^word:]'   => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x30-\x39\x41-\x5A\x5F\x61-\x7A])',
-                '[:^xdigit:]' => '(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF\x30-\x39\x41-\x46\x61-\x66])',
+                '[:^alnum:]'  => '@{Eutf2::not_alnum}',
+                '[:^alpha:]'  => '@{Eutf2::not_alpha}',
+                '[:^ascii:]'  => '@{Eutf2::not_ascii}',
+                '[:^blank:]'  => '@{Eutf2::not_blank}',
+                '[:^cntrl:]'  => '@{Eutf2::not_cntrl}',
+                '[:^digit:]'  => '@{Eutf2::not_digit}',
+                '[:^graph:]'  => '@{Eutf2::not_graph}',
+                '[:^lower:]'  => '@{Eutf2::not_lower}',
+                '[:^print:]'  => '@{Eutf2::not_print}',
+                '[:^punct:]'  => '@{Eutf2::not_punct}',
+                '[:^space:]'  => '@{Eutf2::not_space}',
+                '[:^upper:]'  => '@{Eutf2::not_upper}',
+                '[:^word:]'   => '@{Eutf2::not_word}',
+                '[:^xdigit:]' => '@{Eutf2::not_xdigit}',
 
             }->{$1};
         }
@@ -1307,48 +1387,48 @@ sub _charlist {
                 }
             }
 
-            # range of multiple octet code
+            # range of multiple-octet code
             elsif (length($char[$i-1]) == length($char[$i+1])) {
                 push @charlist, _octets(length($char[$i-1]), $char[$i-1], $char[$i+1], $modifier);
             }
             elsif (length($char[$i-1]) == 1) {
                 if (length($char[$i+1]) == 2) {
                     push @charlist,
-                        _octets(1, $char[$i-1], &maxchar(1), $modifier),
-                        _octets(2, &minchar(2), $char[$i+1], $modifier);
+                        _octets(1, $char[$i-1], maxchar(1),  $modifier),
+                        _octets(2, minchar(2),  $char[$i+1], $modifier);
                 }
                 elsif (length($char[$i+1]) == 3) {
                     push @charlist,
-                        _octets(1, $char[$i-1], &maxchar(1), $modifier),
-                        _octets(2, &minchar(2), &maxchar(2), $modifier),
-                        _octets(3, &minchar(3), $char[$i+1], $modifier);
+                        _octets(1, $char[$i-1], maxchar(1),  $modifier),
+                        _octets(2, minchar(2),  maxchar(2),  $modifier),
+                        _octets(3, minchar(3),  $char[$i+1], $modifier);
                 }
                 elsif (length($char[$i+1]) == 4) {
                     push @charlist,
-                        _octets(1, $char[$i-1], &maxchar(1), $modifier),
-                        _octets(2, &minchar(2), &maxchar(2), $modifier),
-                        _octets(3, &minchar(3), &maxchar(3), $modifier),
-                        _octets(4, &minchar(4), $char[$i+1], $modifier);
+                        _octets(1, $char[$i-1], maxchar(1),  $modifier),
+                        _octets(2, minchar(2),  maxchar(2),  $modifier),
+                        _octets(3, minchar(3),  maxchar(3),  $modifier),
+                        _octets(4, minchar(4),  $char[$i+1], $modifier);
                 }
             }
             elsif (length($char[$i-1]) == 2) {
                 if (length($char[$i+1]) == 3) {
                     push @charlist,
-                        _octets(2, $char[$i-1], &maxchar(2), $modifier),
-                        _octets(3, &minchar(3), $char[$i+1], $modifier);
+                        _octets(2, $char[$i-1], maxchar(2),  $modifier),
+                        _octets(3, minchar(3),  $char[$i+1], $modifier);
                 }
                 elsif (length($char[$i+1]) == 4) {
                     push @charlist,
-                        _octets(2, $char[$i-1], &maxchar(2), $modifier),
-                        _octets(3, &minchar(3), &maxchar(3), $modifier),
-                        _octets(4, &minchar(4), $char[$i+1], $modifier);
+                        _octets(2, $char[$i-1], maxchar(2),  $modifier),
+                        _octets(3, minchar(3),  maxchar(3),  $modifier),
+                        _octets(4, minchar(4),  $char[$i+1], $modifier);
                 }
             }
             elsif (length($char[$i-1]) == 3) {
                 if (length($char[$i+1]) == 4) {
                     push @charlist,
-                        _octets(3, $char[$i-1], &maxchar(3), $modifier),
-                        _octets(4, &minchar(4), $char[$i+1], $modifier);
+                        _octets(3, $char[$i-1], maxchar(3),  $modifier),
+                        _octets(4, minchar(4),  $char[$i+1], $modifier);
                 }
             }
             else {
@@ -1377,13 +1457,6 @@ sub _charlist {
         }
 
         # single character of single octet code
-
-        # \h \v
-        #
-        # P.114 Character Class Shortcuts
-        # in Chapter 7: In the World of Regular Expressions
-        # of ISBN 978-0-596-52010-6 Learning Perl, Fifth Edition
-
         elsif ($char[$i] =~ m/\A (?: \\h ) \z/oxms) {
             push @singleoctet, "\t", "\x20";
             $i += 1;
@@ -1397,7 +1470,7 @@ sub _charlist {
             $i += 1;
         }
 
-        # single character of multiple octet code
+        # single character of multiple-octet code
         else {
             push @charlist, $char[$i];
             $i += 1;
@@ -1524,12 +1597,12 @@ sub charlist_not_qr {
     if (scalar(@charlist) >= 1) {
         if (scalar(@singleoctet) >= 1) {
 
-            # any character other than multiple octet and single octet character class
+            # any character other than multiple-octet and single octet character class
             return '(?!' . join('|', @charlist) . ')(?:(?:[\xC2-\xDF]|[\xE0-\xE0][\xA0-\xBF]|[\xE1-\xEC][\x80-\xBF]|[\xED-\xED][\x80-\x9F]|[\xEE-\xEF][\x80-\xBF]|[\xF0-\xF0][\x90-\xBF][\x80-\xBF]|[\xF1-\xF3][\x80-\xBF][\x80-\xBF]|[\xF4-\xF4][\x80-\x8F][\x80-\xBF])[\x00-\xFF]|[^\x80-\xFF'. join('', @singleoctet) . '])';
         }
         else {
 
-            # any character other than multiple octet character class
+            # any character other than multiple-octet character class
             return '(?!' . join('|', @charlist) . ")(?:$your_char)";
         }
     }
@@ -1858,6 +1931,27 @@ sub _parse_path {
     return $head, $tail;
 }
 
+#
+# ${^PREMATCH}, $PREMATCH, $` the string preceding what was matched
+#
+sub Eutf2::PREMATCH {
+    return $`;
+}
+
+#
+# ${^MATCH}, $MATCH, $& the string that matched
+#
+sub Eutf2::MATCH {
+    return $&;
+}
+
+#
+# ${^POSTMATCH}, $POSTMATCH, $' the string following what was matched
+#
+sub Eutf2::POSTMATCH {
+    return $';
+}
+
 #
 # UTF-2 character to order (with parameter)
 #
@@ -2084,7 +2178,12 @@ Because the UTF2.pm automatically uses this module, you need not use directly.
 
 =head1 BUGS AND LIMITATIONS
 
-Please patches and report problems to author are welcome.
+I have tested and verified this software using the best of my ability.
+However, a software containing much regular expression is bound to contain
+some bugs. Thus, if you happen to find a bug that's in UTF2 software and not
+your own program, you can try to reduce it to a minimal test case and then
+report it to the following author's address. If you have an idea that could
+make this a more useful tool, please let everyone share it.
 
 =head1 HISTORY
 
@@ -2141,6 +2240,8 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   If the pattern contains parentheses, then the substring matched by each pair of
   parentheses is included in the resulting list, interspersed with the fields that
   are ordinarily returned.
+  Unlike Perl4, you cannot force the split into @_ by using ?? as the pattern
+  delimiters, it only returns the list value.
 
 =item Transliteration
 
